package betterwithmods.common.advancements;


import betterwithmods.lib.ModLib;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonObject;
import net.minecraft.advancements.ICriterionTrigger;
import net.minecraft.advancements.PlayerAdvancements;
import net.minecraft.advancements.critereon.AbstractCriterionInstance;
import net.minecraft.advancements.critereon.ItemPredicate;
import net.minecraft.advancements.critereon.MinMaxBounds;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.item.ItemStack;
import net.minecraft.util.ResourceLocation;

import javax.annotation.Nonnull;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class InfernalEnchantedTrigger implements ICriterionTrigger<InfernalEnchantedTrigger.Instance> {
    private static final ResourceLocation ID = new ResourceLocation(ModLib.MODID, "infernal_enchant");
    private final Map<PlayerAdvancements, InfernalEnchantedTrigger.Listeners> listeners = Maps.newHashMap();

    @Nonnull
    public ResourceLocation func_192163_a() {
        return ID;
    }

    public void func_192165_a(@Nonnull PlayerAdvancements playerAdvancementsIn, @Nonnull ICriterionTrigger.Listener<InfernalEnchantedTrigger.Instance> listener) {
        InfernalEnchantedTrigger.Listeners enchanteditemtrigger$listeners = this.listeners.get(playerAdvancementsIn);

        if (enchanteditemtrigger$listeners == null) {
            enchanteditemtrigger$listeners = new InfernalEnchantedTrigger.Listeners(playerAdvancementsIn);
            this.listeners.put(playerAdvancementsIn, enchanteditemtrigger$listeners);
        }

        enchanteditemtrigger$listeners.add(listener);
    }

    public void func_192164_b(@Nonnull PlayerAdvancements playerAdvancementsIn, @Nonnull ICriterionTrigger.Listener<InfernalEnchantedTrigger.Instance> listener) {
        InfernalEnchantedTrigger.Listeners enchanteditemtrigger$listeners = this.listeners.get(playerAdvancementsIn);

        if (enchanteditemtrigger$listeners != null) {
            enchanteditemtrigger$listeners.remove(listener);

            if (enchanteditemtrigger$listeners.isEmpty()) {
                this.listeners.remove(playerAdvancementsIn);
            }
        }
    }

    public void func_192167_a(@Nonnull PlayerAdvancements playerAdvancementsIn) {
        this.listeners.remove(playerAdvancementsIn);
    }

    /**
     * Deserialize a ICriterionInstance of this trigger from the data in the JSON.
     */
    @Nonnull
    public InfernalEnchantedTrigger.Instance func_192166_a(@Nonnull JsonObject json, @Nonnull JsonDeserializationContext context) {
        ItemPredicate itempredicate = ItemPredicate.func_192492_a(json.get("item"));
        MinMaxBounds minmaxbounds = MinMaxBounds.func_192515_a(json.get("levels"));
        return new InfernalEnchantedTrigger.Instance(itempredicate, minmaxbounds);
    }

    public void trigger(EntityPlayerMP player, ItemStack item, int levelsSpent) {
        InfernalEnchantedTrigger.Listeners enchanteditemtrigger$listeners = this.listeners.get(player.func_192039_O());

        if (enchanteditemtrigger$listeners != null) {
            enchanteditemtrigger$listeners.trigger(item, levelsSpent);
        }
    }

    public static class Instance extends AbstractCriterionInstance {
        private final ItemPredicate item;
        private final MinMaxBounds levels;

        public Instance(ItemPredicate item, MinMaxBounds levels) {
            super(InfernalEnchantedTrigger.ID);
            this.item = item;
            this.levels = levels;
        }

        public boolean test(ItemStack item, int levelsIn) {
            if (!this.item.func_192493_a(item)) {
                return false;
            } else {
                return this.levels.func_192514_a((float) levelsIn);
            }
        }
    }

    static class Listeners {
        private final PlayerAdvancements playerAdvancements;
        private final Set<ICriterionTrigger.Listener<InfernalEnchantedTrigger.Instance>> listeners = Sets.newHashSet();

        public Listeners(PlayerAdvancements playerAdvancementsIn) {
            this.playerAdvancements = playerAdvancementsIn;
        }

        public boolean isEmpty() {
            return this.listeners.isEmpty();
        }

        public void add(ICriterionTrigger.Listener<InfernalEnchantedTrigger.Instance> listener) {
            this.listeners.add(listener);
        }

        public void remove(ICriterionTrigger.Listener<InfernalEnchantedTrigger.Instance> listener) {
            this.listeners.remove(listener);
        }

        public void trigger(ItemStack item, int levelsIn) {
            List<ICriterionTrigger.Listener<InfernalEnchantedTrigger.Instance>> list = null;

            for (ICriterionTrigger.Listener<InfernalEnchantedTrigger.Instance> listener : this.listeners) {
                if (listener.func_192158_a().test(item, levelsIn)) {
                    if (list == null) {
                        list = Lists.newArrayList();
                    }

                    list.add(listener);
                }
            }

            if (list != null) {
                for (ICriterionTrigger.Listener<InfernalEnchantedTrigger.Instance> listener1 : list) {
                    listener1.func_192159_a(this.playerAdvancements);
                }
            }
        }
    }
}
