package betterwithmods.common.blocks.mechanical;

import betterwithmods.common.BWMBlocks;
import betterwithmods.common.blocks.mechanical.mech_machine.BlockMechMachine;
import betterwithmods.common.registry.BellowsManager;
import betterwithmods.common.tile.TileBellows;
import betterwithmods.lib.ModLib;
import betterwithmods.library.utils.DirUtils;
import betterwithmods.module.internal.SoundRegistry;
import betterwithmods.util.WorldUtils;
import net.minecraft.block.Block;
import net.minecraft.block.SoundType;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.state.BlockFaceShape;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.init.Blocks;
import net.minecraft.init.SoundEvents;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraft.world.storage.loot.LootTableList;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.List;
import java.util.Optional;
import java.util.Random;

import static betterwithmods.library.utils.DirUtils.AXIS;

public class BlockBellows extends BlockMechMachine {

    public static final ResourceLocation BELLOWS = LootTableList.func_186375_a(new ResourceLocation(ModLib.MODID, "block/bellows"));

    private final float scale = 1 / 4f;
    private final AxisAlignedBB lift = new AxisAlignedBB(0, 0.6875F, 0, 1, 1, 1);


    public BlockBellows() {
        super(Material.field_151575_d, BELLOWS);
        this.func_149675_a(true);
        this.func_149711_c(2.0F);
        this.func_180632_j(func_176223_P().func_177226_a(DirUtils.HORIZONTAL, EnumFacing.SOUTH).func_177226_a(ACTIVE, true));
        this.func_149672_a(SoundType.field_185848_a);
    }

    @Override
    public int func_149738_a(World world) {
        return 37;
    }

    @Override
    public void nextState(World world, BlockPos pos, IBlockState state) {
        world.func_175656_a(pos, state.func_177231_a(DirUtils.HORIZONTAL).func_177226_a(ACTIVE, false));
    }

    @Override
    public boolean func_149662_c(IBlockState state) {
        return state.func_177230_c() == this && !state.func_177229_b(ACTIVE);
    }

    @Override
    public boolean func_149686_d(IBlockState state) {
        return state.func_177230_c() == this && !state.func_177229_b(ACTIVE);
    }

    @Nonnull
    @Deprecated
    @Override
    public IBlockState getStateForPlacement(@Nonnull World world, @Nonnull BlockPos pos, @Nonnull EnumFacing side, float flX, float flY, float flZ,
                                            int meta, @Nonnull EntityLivingBase living, EnumHand hand) {
        return super.getStateForPlacement(world, pos, side, flX, flY, flZ, meta, living, hand).func_177226_a(DirUtils.HORIZONTAL, living.func_174811_aO()).func_177226_a(ACTIVE, false);
    }

    public EnumFacing getFacing(IBlockState state) {
        return state.func_177229_b(DirUtils.HORIZONTAL);
    }

    @Nonnull
    @Override
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess world, BlockPos pos) {
        if (state.func_177229_b(ACTIVE))
            return new AxisAlignedBB(0.0F, 0.0F, 0.0F, 1.0F, 0.6875F, 1.0F);
        return field_185505_j;
    }

    @Override
    public void onChangeActive(World world, BlockPos pos, boolean active) {
        world.func_180497_b(pos, this, func_149738_a(world), 5);
        if (active) {
            world.func_184133_a(null, pos, SoundRegistry.BLOCK_WOOD_BELLOW, SoundCategory.BLOCKS, 0.7F, world.field_73012_v.nextFloat() * 0.25F + 2.5F);
            blow(world, pos);
        } else {
            world.func_184133_a(null, pos, SoundRegistry.BLOCK_WOOD_BELLOW, SoundCategory.BLOCKS, 0.2F, world.field_73012_v.nextFloat() * 0.25F + 2.5F);
        }
        liftCollidingEntities(world, pos);
    }

    @Override
    public void func_189540_a(IBlockState state, World world, BlockPos pos, Block block, BlockPos other) {
        withTile(world, pos).ifPresent(TileBellows::onChange);
    }

    @Override
    public void func_180650_b(World world, BlockPos pos, IBlockState state, Random rand) {
        withTile(world, pos).ifPresent(TileBellows::onChange);
    }

    @Override
    public void overpower(World world, BlockPos pos) {
        if (doesOverpower()) {


            world.func_184133_a(null, pos, SoundEvents.field_187539_bB, SoundCategory.BLOCKS, 0.3F,
                    world.field_73012_v.nextFloat() * 0.1F + 0.45F);
            world.func_175698_g(pos);
        }
    }

    public void blow(World world, BlockPos pos) {
        stokeFlames(world, pos);
        blowItems(world, pos);
    }

    public void blowItems(World world, BlockPos pos) {
        EnumFacing facing = getFacing(world.func_180495_p(pos));
        BlockPos pos2 = pos.func_177967_a(facing, 4);
        AxisAlignedBB box = new AxisAlignedBB(pos.func_177958_n(), pos.func_177956_o(), pos.func_177952_p(), pos2.func_177958_n() + 1, pos2.func_177956_o() + 1, pos2.func_177952_p() + 1);

        List<EntityItem> items = world.func_72872_a(EntityItem.class, box);
        for (EntityItem item : items) {
            blowItem(pos, facing, item);
        }
    }

    public void blowItem(BlockPos pos, EnumFacing facing, EntityItem item) {
        double x = 0, z = 0;

        if (WorldUtils.getDistance(pos, item.func_180425_c()) > BellowsManager.getWeight(item.func_92059_d()))
            return;
        switch (facing.func_176740_k()) {
            case X:
                x += facing.func_176743_c().func_179524_a();
                break;
            case Z:
                z += facing.func_176743_c().func_179524_a();
                break;
        }

        item.func_70024_g(x * scale, 0, z * scale);
    }

    //TODO clean this up
    private void stokeFlames(World world, BlockPos pos) {
        EnumFacing dir = getFacing(world.func_180495_p(pos));
        EnumFacing dirLeft = DirUtils.rotateFacingAroundY(getFacing(world.func_180495_p(pos)), false);
        EnumFacing dirRight = DirUtils.rotateFacingAroundY(getFacing(world.func_180495_p(pos)), true);

        for (int i = 0; i < 3; i++) {
            BlockPos dirPos = pos.func_177967_a(dir, 1 + i);
            //
            Block target = world.func_180495_p(dirPos).func_177230_c();

            if (target == Blocks.field_150480_ab || target == BWMBlocks.STOKED_FLAME)
                stokeFire(world, dirPos);
            else if (!world.func_175623_d(dirPos))
                break;

            BlockPos posLeft = dirPos.func_177972_a(dirLeft);

            Block targetLeft = world.func_180495_p(posLeft).func_177230_c();
            if (targetLeft == Blocks.field_150480_ab || targetLeft == BWMBlocks.STOKED_FLAME)
                stokeFire(world, posLeft);

            BlockPos posRight = dirPos.func_177972_a(dirRight);

            Block targetRight = world.func_180495_p(posRight).func_177230_c();
            if (targetRight == Blocks.field_150480_ab || targetRight == BWMBlocks.STOKED_FLAME)
                stokeFire(world, posRight);
        }
    }

    //TODO clean this up too
    private void stokeFire(World world, BlockPos pos) {
        BlockPos down = pos.func_177977_b();

        if (world.func_180495_p(down).func_177230_c() == BWMBlocks.HIBACHI) {
            int flag = (world.func_180495_p(pos).func_177230_c() == BWMBlocks.STOKED_FLAME) ? 4 : 3;
            world.func_180501_a(pos, BWMBlocks.STOKED_FLAME.func_176223_P(), flag);
        } else {
            world.func_175698_g(pos);
        }
    }

    private void liftCollidingEntities(World world, BlockPos pos) {
        List<Entity> list = world.func_72872_a(Entity.class, lift.func_186670_a(pos));
        float extendedY = pos.func_177956_o() + 1;
        if (list.size() > 0) {
            for (Entity entity : list) {
                if (!entity.field_70128_L && (entity.func_70104_M() || entity instanceof EntityItem)) {
                    double tempY = entity.func_174813_aQ().field_72338_b;
                    if (tempY < extendedY) {
                        double entityOffset = extendedY - tempY;
                        entity.func_70107_b(entity.field_70165_t, entity.field_70163_u + entityOffset, entity.field_70161_v);
                    }
                }
            }
        }
    }

    @Override
    public int func_176201_c(IBlockState state) {
        int facing = state.func_177229_b(DirUtils.HORIZONTAL).func_176736_b();
        int active = isActive(state) ? 1 : 0;
        return facing << 1 | active;
    }

    @Nonnull
    @Override
    public IBlockState func_176203_a(int meta) {
        return this.func_176223_P().func_177226_a(ACTIVE, (meta & 1) == 1).func_177226_a(DirUtils.HORIZONTAL, EnumFacing.func_176731_b(meta >> 1));
    }

    @Override
    public IProperty<?>[] getProperties() {
        return new IProperty[]{DirUtils.HORIZONTAL, ACTIVE};
    }



    public Optional<TileBellows> withTile(World world, BlockPos pos) {
        return Optional.ofNullable(getTile(world, pos));
    }

    public TileBellows getTile(World world, BlockPos pos) {
        TileEntity tile = world.func_175625_s(pos);
        if (tile instanceof TileBellows)
            return (TileBellows) tile;
        return null;
    }

    @Nonnull
    @Override
    public BlockFaceShape func_193383_a(IBlockAccess worldIn, IBlockState state, BlockPos pos, EnumFacing face) {
        return face != EnumFacing.DOWN && state.func_177229_b(ACTIVE) ? BlockFaceShape.UNDEFINED : BlockFaceShape.SOLID;
    }

    @Override
    public boolean rotates() {
        return true;
    }

    @Override
    public boolean hasTileEntity(IBlockState state) {
        return true;
    }

    @Nullable
    @Override
    public TileEntity createTileEntity(@Nonnull World world, @Nonnull IBlockState state) {
        return new TileBellows();
    }
}
