package betterwithmods.common.container.other;

import betterwithmods.client.gui.GuiInfernalEnchanter;
import betterwithmods.common.BWMItems;
import betterwithmods.common.items.ItemArcaneScroll;
import betterwithmods.common.tile.TileInfernalEnchanter;
import betterwithmods.lib.ModLib;
import betterwithmods.library.common.container.ContainerTile;
import betterwithmods.library.common.container.SlotItemHandlerFiltered;
import betterwithmods.library.common.container.SlotTransformation;
import betterwithmods.library.common.inventory.SimpleStackHandler;
import betterwithmods.library.utils.GuiUtils;
import betterwithmods.module.general.InfernalEnchanting;
import betterwithmods.module.internal.AdvancementRegistry;
import betterwithmods.util.InfernalEnchantment;
import net.minecraft.client.gui.inventory.GuiContainer;
import net.minecraft.enchantment.Enchantment;
import net.minecraft.enchantment.EnchantmentHelper;
import net.minecraft.entity.effect.EntityLightningBolt;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.Ingredient;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.SoundCategory;
import net.minecraftforge.items.IItemHandler;

import java.util.Arrays;
import java.util.Set;

/**
 * Created by primetoxinz on 9/11/16.
 */

public class ContainerInfernalEnchanter extends ContainerTile<TileInfernalEnchanter> {

    private static final ResourceLocation PROPERTY_BOOKCASES = new ResourceLocation(ModLib.MODID, "bookcases");
    private static final ResourceLocation SLOT_SCROLLS = new ResourceLocation(ModLib.MODID, "scrolls");
    private static final ResourceLocation SLOT_ENCHANT = new ResourceLocation(ModLib.MODID, "enchant");
    private static Ingredient SCROLLS = Ingredient.func_193367_a(BWMItems.ARCANE_SCROLL);
    private int[] enchantLevels;

    public ContainerInfernalEnchanter(TileInfernalEnchanter tile, EntityPlayer player) {
        super(tile, player);

        this.enchantLevels = new int[5];
        Arrays.fill(enchantLevels, -1);

        addIntProperty(PROPERTY_BOOKCASES, tile::getBookcaseCount);
        for (int i = 0; i < this.enchantLevels.length; i++) {
            int finalI = i;
            addIntProperty(getLevelName(i), () -> this.enchantLevels[finalI]);
        }

        SimpleStackHandler inventory = tile.inventory.listener((handler, slot) -> onContextChanged(handler));

        GuiUtils.createPlayerSlots(player, this, 8, 129, 8, 187);
        GuiUtils.createContainerSlots(SLOT_SCROLLS, this, inventory, 1, 1, 0, 17, 37, (inv, i, x, y) -> new SlotItemHandlerFiltered(inv, i, x, y, SCROLLS));
        GuiUtils.createContainerSlots(SLOT_ENCHANT, this, inventory, 1, 1, 1, 17, 75);

        addSlotTransformations(new SlotTransformation(GuiUtils.SLOTS_FULL_PLAYER_INVENTORY, SLOT_ENCHANT));
        addSlotTransformations(new SlotTransformation(GuiUtils.SLOTS_FULL_PLAYER_INVENTORY, SLOT_SCROLLS, SCROLLS));
        addSlotTransformations(new SlotTransformation(SLOT_ENCHANT, GuiUtils.SLOTS_FULL_PLAYER_INVENTORY));
        addSlotTransformations(new SlotTransformation(SLOT_SCROLLS, GuiUtils.SLOTS_FULL_PLAYER_INVENTORY));

        onContextChanged(inventory);
    }

    private static ResourceLocation getLevelName(int i) {
        return new ResourceLocation(ModLib.MODID, "enchant_level_" + i);
    }

    public int getEnchantLevel(int i) {
        ResourceLocation l = getLevelName(i);
        return getPropertyValue(l, 0);
    }

    private boolean areValidItems(ItemStack scroll, ItemStack item) {
        if (!scroll.func_190926_b() && !item.func_190926_b()) {

            Enchantment enchantment = ItemArcaneScroll.getEnchantment(scroll);
            if (enchantment == null)
                return false;

            Set<Enchantment> enchantments = EnchantmentHelper.func_82781_a(item).keySet();
            if (enchantments.contains(enchantment))
                return false;
            for (Enchantment e : enchantments) {
                if (e != null && !e.func_191560_c(enchantment))
                    return false;
            }
            enchantment = new InfernalEnchantment(enchantment);
            return item.func_77973_b().canApplyAtEnchantingTable(item, enchantment);
        }
        return false;
    }

    private void onContextChanged(IItemHandler handler) {
        ItemStack scroll = handler.getStackInSlot(0);
        ItemStack item = handler.getStackInSlot(1);
        if (areValidItems(scroll, item)) {
            Enchantment enchantment = ItemArcaneScroll.getEnchantment(scroll);
            int enchantCount = EnchantmentHelper.func_82781_a(item).size();
            for (int levelIndex = 1; levelIndex <= enchantLevels.length; levelIndex++) {
                enchantLevels[levelIndex - 1] = getEnchantCost(levelIndex, enchantment, enchantCount);
            }
        } else {
            Arrays.fill(enchantLevels, -1);
        }
        func_75142_b();
    }

    private int getEnchantCost(int levelIndex, Enchantment enchantment, int enchantCount) {
        if (enchantment == null || levelIndex > InfernalEnchanting.getMaxLevel(enchantment)) {
            return -1;
        } else {
            double max = Math.min(InfernalEnchanting.getMaxLevel(enchantment), enchantLevels.length);
            double multiplier = levelIndex / max;
            return (int) Math.ceil(30.0 * multiplier) + (30 * enchantCount);
        }
    }

    @Override
    public GuiContainer createGui() {
        return new GuiInfernalEnchanter(this);
    }

    public boolean hasLevels(EntityPlayer player, int levelIndex) {
        return player.field_71075_bZ.field_75098_d || player.field_71068_ca >= this.enchantLevels[levelIndex];
    }

    public boolean hasBooks(int levelIndex) {
        return getTile().getBookcaseCount() >= this.enchantLevels[levelIndex];
    }

    @Override
    public boolean func_75140_a(EntityPlayer player, int levelIndex) {
        if (hasLevels(player, levelIndex) && hasBooks(levelIndex)) {
            if (!player.field_70170_p.field_72995_K) {
                ItemStack item = getTile().inventory.getStackInSlot(1);
                ItemStack scroll = getTile().inventory.getStackInSlot(0);
                Enchantment enchantment = ItemArcaneScroll.getEnchantment(scroll);
                if (enchantment != null) {
                    if (!EnchantmentHelper.func_82781_a(item).containsKey(enchantment)) {
                        scroll.func_190918_g(1);
                        item.func_77966_a(enchantment, levelIndex + 1);
                        player.func_192024_a(item, this.enchantLevels[levelIndex]);
                        AdvancementRegistry.INFERNAL_ENCHANTED.trigger((EntityPlayerMP) player, item, this.enchantLevels[levelIndex]);
                        TileEntity tile = getTile();
                        tile.func_145831_w().func_72942_c(new EntityLightningBolt(tile.func_145831_w(), tile.func_174877_v().func_177958_n(), tile.func_174877_v().func_177956_o(), tile.func_174877_v().func_177952_p(), true));
                        tile.func_145831_w().func_184133_a(null, getTile().func_174877_v(), SoundEvents.field_187754_de, SoundCategory.BLOCKS, 1.0F, getTile().func_145831_w().field_73012_v.nextFloat() * 0.1F + 0.9F);
                        onContextChanged(getTile().inventory);
                    }
                }
            }
            return true;
        }
        return false;
    }


}
