package betterwithmods.common.tile;

import betterwithmods.common.registry.KilnStructureManager;
import betterwithmods.common.registry.block.recipe.KilnRecipe;
import betterwithmods.module.internal.RecipeRegistry;
import net.minecraft.block.state.IBlockState;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.ITickable;
import net.minecraft.util.math.BlockPos;

import javax.annotation.Nonnull;

public class TileKiln extends TileDynamic implements ITickable {

    private int cookTicks;
    private int prevProgress;
    private int cookSpeed;

    @Override
    public void func_73660_a() {

        if (!KilnStructureManager.isValidKiln(field_145850_b, field_174879_c)) {
            field_145850_b.func_175656_a(field_174879_c, this.state);
            return;
        }

        BlockPos cookPos = field_174879_c.func_177984_a();
        IBlockState cookState = field_145850_b.func_180495_p(cookPos);
        KilnRecipe recipe = RecipeRegistry.KILN.findRecipe(field_145850_b, cookPos, cookState).orElse(null);
        if (recipe != null) {
            int progress = (int) ((((double) cookTicks) / ((double) recipe.getCookTime())) * 10);
            if (prevProgress != progress) {
                prevProgress = progress;
                field_145850_b.func_175715_c(0, cookPos, prevProgress);
                cookSpeed = calculateSpeed();
            }
            if (recipe.craftRecipe(field_145850_b, cookPos, field_145850_b.field_73012_v, cookState)) {
                field_145850_b.func_175715_c(0, cookPos, -1);
                cookTicks = 0;
                prevProgress = -1;
            } else {
                cookTicks += getCookSpeed();
            }
        } else {
            cookTicks = 0;
            prevProgress = -1;
        }

    }

    public int getCookTicks() {
        return cookTicks;
    }

    public int getCookSpeed() {
        return cookSpeed;
    }

    @Nonnull
    @Override
    public NBTTagCompound func_189515_b(NBTTagCompound compound) {

        compound.func_74768_a("cookTicks", cookTicks);
        compound.func_74768_a("prevProgress", prevProgress);
        return super.func_189515_b(compound);
    }

    @Override
    public void func_145839_a(NBTTagCompound compound) {
        cookTicks = compound.func_74762_e("cookTicks");
        prevProgress = compound.func_74762_e("prevProgress");
        super.func_145839_a(compound);
    }

    private int calculateSpeed() {
        int speed = 0;
        int centerFire = KilnStructureManager.getKiln().getHeat(field_145850_b, field_174879_c);

        for (int xP = -1; xP < 2; xP++) {
            for (int zP = -1; zP < 2; zP++) {
                BlockPos bPos = field_174879_c.func_177982_a(xP, 0, zP);
                int currentFire = KilnStructureManager.getKiln().getHeat(field_145850_b, bPos);
                if (currentFire == centerFire)
                    speed += currentFire;
            }
        }
        return speed / centerFire;
    }


}
