package betterwithmods.module.compat.patchouli;

import betterwithmods.common.registry.KilnStructureManager;
import betterwithmods.lib.ModLib;
import betterwithmods.library.common.modularity.impl.Feature;
import betterwithmods.library.utils.CapabilityUtils;
import betterwithmods.library.utils.InventoryUtils;
import betterwithmods.library.utils.ingredient.NBTIngredient;
import betterwithmods.module.internal.player.PlayerInfo;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.Ingredient;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.EnumParticleTypes;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.SoundEvent;
import net.minecraftforge.event.entity.EntityJoinWorldEvent;
import net.minecraftforge.event.entity.player.EntityItemPickupEvent;
import net.minecraftforge.event.entity.player.PlayerInteractEvent;
import net.minecraftforge.fml.common.event.FMLInitializationEvent;
import net.minecraftforge.fml.common.event.FMLPreInitializationEvent;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.common.registry.ForgeRegistries;
import net.minecraftforge.fml.common.registry.GameRegistry;
import net.minecraftforge.items.IItemHandler;
import vazkii.patchouli.api.IMultiblock;
import vazkii.patchouli.api.PatchouliAPI;

import java.util.UUID;

public class Patchouli extends Feature {

    private static boolean giveJournal;

    public Patchouli() {
    }

    @Override
    public String getDescription() {
        return "Patchouli API registration";
    }

    @Override
    public boolean hasEvent() {
        return true;
    }


    private static IMultiblock kilnTube, kilnCorner;

    @Override
    public void onPreInit(FMLPreInitializationEvent event) {
        giveJournal = loadProperty("Give Journal", true).setComment("give the player the journal on their first respawn and on each new HCSpawn.").get();
    }

    @Override
    public void onInit(FMLInitializationEvent event) {
        PatchouliAPI.IPatchouliAPI api = PatchouliAPI.instance;


        Object[] kilnMatchers = new Object[]{
                'E', api.airMatcher(),
                ' ', api.anyMatcher(),
                'B', api.predicateMatcher(Blocks.field_150336_V, KilnStructureManager::isKilnBlock)
        };

        kilnTube = api.registerMultiblock(
                new ResourceLocation(ModLib.MODID, "kiln_tube"),
                api.makeMultiblock(new String[][]{
                        {"   ", " B ", "   "},
                        {"   ", "BEB", "   "},
                        {"   ", " B ", "   "},
                        {"   ", " 0 ", "   "},
                }, kilnMatchers))
                .setSymmetrical(true);

        kilnCorner = api.registerMultiblock(
                new ResourceLocation(ModLib.MODID, "kiln_corner"),
                api.makeMultiblock(new String[][]{
                        {"   ", " B ", "   "},
                        {"   ", "BE ", " B "},
                        {"   ", " B ", "   "},
                        {"   ", " 0 ", "   "}
                }, kilnMatchers))
                .setSymmetrical(true);

    }

    private static boolean isJournal(ItemStack stack) {
        if (!stack.func_190926_b() && stack.func_77973_b().getRegistryName().equals(new ResourceLocation("patchouli:guide_book"))) {
            NBTTagCompound tag = stack.func_77978_p();
            if (tag != null) {
                String book = tag.func_74779_i("patchouli:book");
                return book.equals("betterwithmods:journal");
            }
        }
        return false;
    }

    private static UUID getOwner(ItemStack stack) {
        if (isJournal(stack)) {
            NBTTagCompound tag = stack.func_77978_p();
            if (tag != null) {
                return UUID.fromString(tag.func_74779_i("betterwithmods:owner"));
            }
        }
        return null;
    }

    @GameRegistry.ObjectHolder("patchouli:guide_book")
    public static final Item GUIDE_BOOK = null;

    @GameRegistry.ObjectHolder("patchouli:book_flip")
    public static final SoundEvent BOOK_FLIP = null;

    public static ItemStack getJournal(EntityPlayer player) {
        if (GUIDE_BOOK == null)
            return ItemStack.field_190927_a;
        ItemStack stack = new ItemStack(GUIDE_BOOK);
        NBTTagCompound tag = new NBTTagCompound();
        String uuid = player.func_110124_au().toString();
        tag.func_74778_a("betterwithmods:owner", uuid);
        tag.func_74778_a("patchouli:book", "betterwithmods:journal");
        stack.func_77982_d(tag);
        return stack;
    }

    public static void giveJournal(EntityPlayer player) {
        if (!giveJournal)
            return;

        ItemStack journal = Patchouli.getJournal(player);
        if (journal.func_190926_b())
            return;
        IItemHandler inv = CapabilityUtils.getFullPlayerInventory(player);
        if (!InventoryUtils.containsIngredient(new NBTIngredient(journal), inv)) {
            InventoryUtils.insert(inv, journal, false);
        }
    }

    @SubscribeEvent
    public void onItemClicked(PlayerInteractEvent.RightClickItem event) {
        ItemStack stack = event.getItemStack();
        if (isJournal(stack)) {
            NBTTagCompound tag = stack.func_77978_p();
            if (tag != null && !tag.func_74764_b("betterwithmods:owner")) {
                tag.func_74778_a("betterwithmods:owner", event.getEntityPlayer().func_110124_au().toString());
            }
        }
    }

    @SubscribeEvent
    public void onEntityItemPickup(EntityItemPickupEvent event) {
        EntityPlayer player = event.getEntityPlayer();
        Ingredient journal = new NBTIngredient(getJournal(player));


        EntityItem entity = event.getItem();
        if (journal.apply(entity.func_92059_d()) && InventoryUtils.containsIngredient(journal, CapabilityUtils.getFullPlayerInventory(player))) {
            if (BOOK_FLIP != null) {
                player.func_130014_f_().func_184133_a(null, entity.func_180425_c(), BOOK_FLIP, SoundCategory.PLAYERS, 1, 1);
            }
            entity.func_130014_f_().func_175718_b(2005, entity.func_180425_c(), 0);

            entity.func_92059_d().func_190918_g(1);
        }
    }


    @SubscribeEvent
    public void onPlayerJoin(EntityJoinWorldEvent event) {
        if (event.getEntity() instanceof EntityPlayer) {
            EntityPlayer player = (EntityPlayer) event.getEntity();
            PlayerInfo info = PlayerInfo.getPlayerInfo(player);
            if (!info.givenManual) {
                giveJournal(player);
            }
        }
    }
}

