package betterwithmods.module.conversion.beacons;

import betterwithmods.BetterWithMods;
import betterwithmods.lib.ModLib;
import betterwithmods.lib.TooltipLib;
import betterwithmods.library.utils.LocaleUtils;
import betterwithmods.library.utils.ingredient.blockstate.BlockStateIngredient;
import betterwithmods.module.hardcore.world.spawn.HCSpawn;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumHand;
import net.minecraft.util.NonNullList;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.common.util.INBTSerializable;
import net.minecraftforge.event.entity.living.LivingDeathEvent;
import net.minecraftforge.fml.common.Mod;
import net.minecraftforge.fml.common.eventhandler.EventPriority;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;

import javax.annotation.Nonnull;
import java.awt.*;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

/**
 * Created by primetoxinz on 7/17/17.
 */
@Mod.EventBusSubscriber(modid = ModLib.MODID)
public class SpawnBeaconEffect extends BeaconEffect {

    public static final HashMap<BlockPos, HashSet<BindingPoint>> SPAWN_LIST = Maps.newHashMap();

    public SpawnBeaconEffect() {
        super("spawn", new BlockStateIngredient("blockSoulforgedSteel"), EntityPlayer.class);
        this.setBaseBeamColor(Color.GRAY);
    }


    public static void removeAll(BlockPos pos) {
        SPAWN_LIST.remove(pos);
    }

    public static void addPoint(BlockPos pos, BindingPoint point) {
        if (SPAWN_LIST.containsKey(pos)) {
            HashSet<BindingPoint> points = SPAWN_LIST.get(pos);
            if (!containsEntry(pos, point))
                points.add(point);
            SPAWN_LIST.put(pos, points);
        } else {
            SPAWN_LIST.put(pos, Sets.newHashSet(point));
        }
    }

    @SuppressWarnings("BooleanMethodIsAlwaysInverted")
    public static boolean containsEntry(BlockPos pos, BindingPoint point) {
        if (SPAWN_LIST.containsKey(pos)) {
            HashSet<BindingPoint> points = SPAWN_LIST.get(pos);
            return points.contains(point);
        }
        return false;
    }

    public static BlockPos findSpawn(EntityPlayer player, World world) {
        for (BlockPos pos : SPAWN_LIST.keySet()) {
            if (shouldSpawnHere(pos, player, world)) {
                return pos;
            }
        }
        return null;
    }

    public static boolean shouldSpawnHere(BlockPos beacon, EntityPlayer player, World world) {
        if (SPAWN_LIST.containsKey(beacon)) {
            Set<BindingPoint> points = SPAWN_LIST.get(beacon);
            for (BindingPoint point : points) {
                if (point.canSpawn(beacon, player, world))
                    return true;
            }
        }
        return false;
    }

    @SubscribeEvent(priority = EventPriority.LOWEST)
    public static void findSpawn(LivingDeathEvent event) {
        if (!BetterWithMods.MODULE_LOADER.isFeatureEnabled(HCBeacons.class))
            return;

        if (!(event.getEntity() instanceof EntityPlayerMP))
            return;
        EntityPlayerMP player = (EntityPlayerMP) event.getEntity();

        BlockPos pos = findSpawn(player, player.field_70170_p);
        if (pos != null) {
            //Set spawn up one block so you don't spawn in the beacon.
            HCSpawn.setSpawn(player, pos.func_177984_a());
        }
    }

    @Override
    public boolean onPlayerInteracted(World world, BlockPos pos, int level, EntityPlayer player, EnumHand hand, ItemStack stack) {
        BindingPoint point = new BindingPoint(player, level);

        if (!containsEntry(pos, point)) {
            addPoint(pos, point);
            player.func_146105_b(LocaleUtils.getMessageComponent(ModLib.MODID, TooltipLib.BEACON_SPAWN_SET), true);
        } else {
            player.func_146105_b(LocaleUtils.getMessageComponent(ModLib.MODID, TooltipLib.BEACON_SPAWN_ALREADY_SET), true);
        }

        return true;
    }

    @Override
    public void onBeaconCreate(@Nonnull World world, @Nonnull BlockPos pos, int beaconLevel) {
    }

    @Override
    public void apply(NonNullList<EntityLivingBase> entitiesInRange, @Nonnull World world, @Nonnull BlockPos pos, int beaconLevel) {
    }

    @Override
    public void onBeaconBreak(World world, BlockPos pos, int level) {
    }

    public enum SpawnType {

        LEVEL1(40),
        LEVEL2(160),
        LEVEL3(-1),
        LEVEL4(-2);

        public static SpawnType[] VALUES = values();
        int range;

        SpawnType(int range) {
            this.range = range;
        }

        public boolean inRange(BlockPos beacon, EntityPlayer player, World world) {
            switch (this.range) {
                case -1:
                    return player.field_71093_bK == world.field_73011_w.getDimension();
                case -2:
                    return true;
                default:
                    double d = player.func_70011_f(beacon.func_177958_n(), beacon.func_177956_o(), beacon.func_177952_p());
                    return d <= this.range;
            }
        }
    }

    public static class BindingPoint implements INBTSerializable<NBTTagCompound> {

        private UUID uuid;
        private SpawnType type;


        public BindingPoint(EntityPlayer player, int level) {
            this(player.func_146103_bH().getId(), SpawnType.VALUES[level]);
        }

        public BindingPoint(UUID uuid, SpawnType type) {
            this.uuid = uuid;
            this.type = type;
        }

        public BindingPoint(NBTTagCompound compound) {
            deserializeNBT(compound);
        }

        public boolean canSpawn(BlockPos beacon, EntityPlayer player, World world) {
            TileEntity tile = world.func_175625_s(beacon);
            if (isPlayer(player) && (tile instanceof TileBeacon) && (((TileBeacon) tile).func_191979_s() - 1) == type.ordinal())
                return type.inRange(beacon, player, world);
            return false;
        }

        public boolean isPlayer(EntityPlayer player) {
            return player.func_146103_bH() != null && player.func_146103_bH().getId().equals(uuid);
        }

        @Override
        public boolean equals(Object o) {
            if (o instanceof BindingPoint) {
                return ((BindingPoint) o).uuid.equals(this.uuid) && this.type == ((BindingPoint) o).type;
            }
            return false;
        }

        @Override
        public int hashCode() {
            return uuid.hashCode() ^ type.hashCode();
        }

        @Override
        public NBTTagCompound serializeNBT() {
            NBTTagCompound tag = new NBTTagCompound();
            tag.func_74768_a("type", type.ordinal());
            tag.func_74778_a("uuid", uuid.toString());
            return tag;
        }

        @Override
        public void deserializeNBT(NBTTagCompound nbt) {
            this.type = SpawnType.VALUES[nbt.func_74762_e("type")];
            this.uuid = UUID.fromString(nbt.func_74779_i("uuid"));
        }
    }

}
