package betterwithmods.module.internal;

import betterwithmods.api.tile.dispenser.IBehaviorCollect;
import betterwithmods.api.tile.dispenser.IBehaviorEntity;
import betterwithmods.common.BWMBlocks;
import betterwithmods.common.BWMItems;
import betterwithmods.common.blocks.BlockMiningCharge;
import betterwithmods.common.blocks.BlockUnfiredPottery;
import betterwithmods.common.items.ItemMaterial;
import betterwithmods.common.registry.advanceddispenser.*;
import betterwithmods.library.common.modularity.impl.RequiredFeature;
import betterwithmods.library.lib.ReflectionLib;
import betterwithmods.library.utils.GlobalUtils;
import betterwithmods.library.utils.InventoryUtils;
import betterwithmods.library.utils.ListUtils;
import betterwithmods.library.utils.ingredient.EntityIngredient;
import betterwithmods.library.utils.ingredient.collections.BlockIngredientMap;
import betterwithmods.library.utils.ingredient.collections.EntityIngredientMap;
import betterwithmods.library.utils.ingredient.collections.IngredientMap;
import com.google.common.collect.Sets;
import net.minecraft.block.state.IBlockState;
import net.minecraft.dispenser.IBehaviorDispenseItem;
import net.minecraft.entity.EntityAgeable;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.entity.passive.EntitySheep;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.init.SoundEvents;
import net.minecraft.inventory.IInventory;
import net.minecraft.inventory.InventoryHelper;
import net.minecraft.item.Item;
import net.minecraft.item.ItemMinecart;
import net.minecraft.item.ItemStack;
import net.minecraft.util.NonNullList;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.SoundCategory;
import net.minecraftforge.fml.common.event.FMLInitializationEvent;
import net.minecraftforge.fml.relauncher.ReflectionHelper;

public class AdvancedDispenserRegistry extends RequiredFeature {

    public static IngredientMap<IBehaviorDispenseItem> BLOCK_DISPENSER_REGISTRY = new IngredientMap<>(new BehaviorDefaultDispenseBlock());
    public static BlockIngredientMap<IBehaviorCollect> BLOCK_COLLECT_REGISTRY = new BlockIngredientMap<>(new BehaviorBreakBlock());
    public static EntityIngredientMap<IBehaviorEntity> ENTITY_COLLECT_REGISTRY = new EntityIngredientMap<>(new BehaviorEntity());


    @Override
    public void onInit(FMLInitializationEvent event) {
        BLOCK_DISPENSER_REGISTRY.put(Items.field_151107_aW, new BehaviorDiodeDispense());
        BLOCK_DISPENSER_REGISTRY.put(Items.field_151132_bS, new BehaviorDiodeDispense());
        BLOCK_DISPENSER_REGISTRY.put(BWMItems.DYNAMITE, new DispenserBehaviorDynamite());
        BLOCK_DISPENSER_REGISTRY.put(BWMBlocks.MINING_CHARGE, BlockMiningCharge::dispenser);
        BLOCK_DISPENSER_REGISTRY.put(Items.field_151119_aD, new DispenserBlockOverride(BlockUnfiredPottery.getStack(BlockUnfiredPottery.Type.BRICK)));
        BLOCK_DISPENSER_REGISTRY.put(ItemMaterial.getItem(ItemMaterial.EnumMaterial.NETHER_SLUDGE), new DispenserBlockOverride(BlockUnfiredPottery.getStack(BlockUnfiredPottery.Type.NETHER_BRICK)));

        BLOCK_COLLECT_REGISTRY.put(Blocks.field_150348_b, new BehaviorSilkTouch());
        BLOCK_COLLECT_REGISTRY.put(Blocks.field_150364_r, new BehaviorSilkTouch());
        BLOCK_COLLECT_REGISTRY.put(Blocks.field_150363_s, new BehaviorSilkTouch());

        //Dispenser Minecarts
        IBehaviorDispenseItem MINECART_DISPENSER_BEHAVIOR = ReflectionHelper.getPrivateValue(ItemMinecart.class, null, ReflectionLib.MINECART_DISPENSER_BEHAVIOR);
        for (Item minecart : Sets.newHashSet(Items.field_151143_au, Items.field_151108_aI, Items.field_151109_aJ, Items.field_151140_bW, Items.field_151142_bV, Items.field_151095_cc)) {
            BLOCK_DISPENSER_REGISTRY.put(minecart, MINECART_DISPENSER_BEHAVIOR);
        }

        IBehaviorEntity MINECART_COLLECT_BEHAVIOR = (world, pos, entity, stack) -> {
            EntityMinecart minecart = (EntityMinecart) entity;

            if (minecart instanceof IInventory) {
                InventoryHelper.func_180176_a(world, minecart, (IInventory) minecart);
            }

            IBlockState state = minecart.func_174897_t();
            if (state == Blocks.field_150470_am.func_176223_P())
                state = Blocks.field_150460_al.func_176223_P();
            ItemStack tile = GlobalUtils.getStackFromState(state);
            minecart.func_70106_y();

            return ListUtils.asNonnullList(new ItemStack(Items.field_151143_au), tile);
        };

        for (String name : Sets.newHashSet("commandblock_minecart", "minecart", "chest_minecart", "furnace_minecart", "tnt_minecart", "hopper_minecart", "spawner_minecart")) {
            ResourceLocation loc = new ResourceLocation(name);
            ENTITY_COLLECT_REGISTRY.put(new EntityIngredient(loc), MINECART_COLLECT_BEHAVIOR);
        }


        ResourceLocation SHEEP = new ResourceLocation("minecraft:sheep");
        ENTITY_COLLECT_REGISTRY.put(new EntityIngredient(SHEEP), (world, pos, entity, stack) -> {
            EntitySheep sheep = (EntitySheep) entity;
            if (sheep.isShearable(new ItemStack(Items.field_151097_aZ), world, pos)) {
                return ListUtils.asNonnullList(sheep.onSheared(new ItemStack(Items.field_151097_aZ), world, pos, 0));
            }
            return NonNullList.func_191196_a();
        });

        ResourceLocation CHICKEN = new ResourceLocation("minecraft:chicken");
        ENTITY_COLLECT_REGISTRY.put(new EntityIngredient(CHICKEN), (world, pos, entity, stack) -> {
            if (((EntityAgeable) entity).func_70631_g_())
                return NonNullList.func_191196_a();
            InventoryUtils.ejectStackWithOffset(world, pos, new ItemStack(Items.field_151008_G, 1 + world.field_73012_v.nextInt(2)));
            world.func_184133_a(null, pos, SoundEvents.field_187666_Z, SoundCategory.NEUTRAL, 0.75F, 1.0F);
            entity.func_70106_y();
            return ListUtils.asNonnullList(new ItemStack(Items.field_151110_aK));
        });

        ResourceLocation COW = new ResourceLocation("minecraft:cow");
        ENTITY_COLLECT_REGISTRY.put(new EntityIngredient(COW), (world, pos, entity, stack) -> {
            if (((EntityAgeable) entity).func_70631_g_())
                return NonNullList.func_191196_a();
            if (stack.func_77969_a(new ItemStack(Items.field_151133_ar))) {
                stack.func_190918_g(1);
                world.func_184133_a(null, pos, SoundEvents.field_187564_an, SoundCategory.BLOCKS, 1.0F, 1.0F);

                InventoryUtils.ejectStackWithOffset(world, pos, new ItemStack(Items.field_151117_aB));
            }
            return NonNullList.func_191196_a();
        });
    }
}
